#!/bin/bash

AMAZON_API='169.254.169.254'
IPSET_TABLE='BLOCKED_IP'

__LOGGER(){

	echo -e "$1" | logger -t pbx-cloud-init

}


__CHECK_IPTABLES_RULE(){

	if [ -z "$(/sbin/iptables -vnL INPUT --line-numbers | grep -w '^1 ' | grep -w "${IPSET_TABLE}")" ]; then
		/sbin/iptables -I INPUT 1 -m set --match-set ${IPSET_TABLE} src -j DROP
	fi

}


__ALLOC_IPSET() {

	local SET="$1"
	local TABLE="$2"

	[ -z "$(/sbin/ipset list -n | grep -ow $TABLE)" ] && /sbin/ipset create $TABLE $SET

}


__SWAP_IPSET() {

	local TABLE="$1"
	local TABLE_TMP="$2"

	/sbin/ipset -W "$TABLE_TMP" "$TABLE"
	/sbin/ipset destroy "$TABLE_TMP"

}


__FIREWALL(){

	local ip_list=$(curl -s --connect-timeout 5 https://wmp.wildix.com/firewall/ips | grep -Eo "([0-9]{1,3}[\.]){3}[0-9]{1,3}")

	__ALLOC_IPSET "hash:net" "${IPSET_TABLE}"
	__ALLOC_IPSET "hash:net" "${IPSET_TABLE}_TMP"
	__CHECK_IPTABLES_RULE
	while read IP; do
		[ "$IP" ] && /sbin/ipset add "${IPSET_TABLE}_TMP" "$IP"
	done <<< $ip_list
	__SWAP_IPSET "${IPSET_TABLE}" "${IPSET_TABLE}_TMP"
	
}


__IS_AMAZON_ALIVE() {

	if [ ! "$(ip r | grep -o "default")" ]; then
		timeout 10 dhclient eth0
	fi

	[ "$(curl --connect-timeout 10 -s http://$AMAZON_API/latest/meta-data/mac/)" ] && echo "1" || echo "0"

}


__EC2_GET_TAG(){

	## main check
	if [ ! -f '/root/.aws/config' ]; then
		echo "AWS not configured for getting TAGs"
		return 10 # aws not configured. Error code 10.
	elif [ ! -f '/usr/bin/aws' ]; then
		echo "AWS not installed on this system"
		return 10 # aws not installed. Error code 10.
	fi

	## define vars
	local KEY_ID="$1"
	local HOME='/root'
	local TMP_FILE="$(mktemp)"
	local INSTANCE_ID="$(ec2metadata --instance-id)"
	local REGION_ID="$(curl --connect-timeout 10 -s http://$AMAZON_API/latest/dynamic/instance-identity/document | awk -F '"' '$2 ~ "region" {print $4}')"
	declare -A ARR_TAGS

	## fill the array with all possible keys/values
	aws ec2 describe-tags --filters "Name=resource-id,Values=$INSTANCE_ID" --region=$REGION_ID --output=text > $TMP_FILE

	## add tags to array
	while read line; do
		local key="$(awk '{print $2}' <<< $line)"
		local value="$(awk '{print $5}' <<< $line)"
		ARR_TAGS[$key]="$value"
	done < $TMP_FILE

	## print tags
	if [ "$KEY_ID" ]; then
		echo "${ARR_TAGS[$KEY_ID]}"
	else
		for i in "${!ARR_TAGS[@]}"; do
			echo -ne $i " "
			echo "${ARR_TAGS[$i]}"
		done
	fi

	## cleanup
	rm -f $TMP_FILE

}


__WRITE_NET_CONF() {

	## delete anyway this rule
	rm -f /etc/systemd/network/*.link

	case "$1" in

		add|ADD)
			## write config in file
			cat << 'END' > /etc/network/interfaces
# The loopback network interface
auto lo
iface lo inet loopback

# The primary network interface
iface eth0 inet manual
iface eth0 inet6 manual

auto eth1
allow-hotplug eth1
iface eth1 inet dhcp
iface eth1 inet6 manual
END
			echo -e "[Match]\nMACAddress=$2\n\n[Link]\nName=eth1" > $3
			__LOGGER "Network configuration written in systemd ---> reboot..."
			sync && reboot && exit 1
		;;

		*)
			## write config in file
			cat << 'END' > /etc/network/interfaces
# The loopback network interface
auto lo
iface lo inet loopback

# The primary network interface
auto eth0
allow-hotplug eth0
iface eth0 inet dhcp
iface eth0 inet6 manual
END
			__LOGGER "Default network configuration written in systemd ---> reboot..."
			sync && reboot && exit 1
		;;

	esac

}


__SET_SERIAL(){

	[ -f '/etc/udev/rules.d/70-persistent-net.rules' ] && rm -f /etc/udev/rules.d/70-persistent-net.rules

	local link_eth1='/etc/systemd/network/01-eth1.link'
	[ -f "$link_eth1" ] && local mac_systemd="$(awk -F '=' '$1 ~ "MACAddress" {print $2}' $link_eth1)"

	if [ "$(__IS_AMAZON_ALIVE)" == "1" ] ; then
		local mac_amazon="$(curl --connect-timeout 10 -s http://$AMAZON_API/latest/meta-data/mac/)"
		local mac_meta="$(__EC2_GET_TAG pbxSerial)"
	fi

	__LOGGER "mac_meta: ${mac_meta}\nmac_amazon: ${mac_amazon}\nmac_systemd: ${mac_systemd}"

	## add rule for rename eth0 to eth1
	if [[ "$mac_amazon" =~ ^[0-9a-zA-Z]{1,2}\:[0-9a-zA-Z]{1,2}\:[0-9a-zA-Z]{1,2}\:[0-9a-zA-Z]{1,2}\:[0-9a-zA-Z]{1,2}\:[0-9a-zA-Z]{1,2}$ ]]; then
		if [ "$mac_amazon" != "$mac_systemd" ]; then
			__WRITE_NET_CONF "add" "$mac_amazon" "$link_eth1"
		else
			__LOGGER "Network configuration correct ---> continues"
		fi
	else
		__WRITE_NET_CONF
	fi

	## add fake interface
	##/sbin/modprobe dummy
	ip link add dummy0 type dummy
	if [ "$(ip a | grep -wo dummy0)" ]; then
		[ "$mac_meta" ] && /sbin/ifconfig dummy0 hw ether "$mac_meta"
		/sbin/ip l s name eth0 dev dummy0
		/sbin/ip l s eth0 up
	fi

}


__SET_SSH_KEY(){

	local SSH_KEY="$(aws secretsmanager get-secret-value --region eu-central-1 --secret-id pbx_ssh_key --query SecretString --output text | cut -d: -f2 | tr -d \"})"
	local KEY_FILE='/etc/skel/.ssh/authorized_keys'

	## main check
	if [ ! "$SSH_KEY" ]; then
        __LOGGER "ERROR: can't get ssh key ---> continues"
        return 1
    elif [ "$(echo "${SSH_KEY}" > /tmp/ssh-key.pub; \
        ssh-keygen -lf /tmp/ssh-key.pub &> /dev/null; \
        echo "$?")" -ne 0 ]; then
        __LOGGER "ERROR: key file is currupted."
        return 1
    fi


	## check ssh-key
	if [ -f "$KEY_FILE" ]; then
		if [ ! "$(grep "$SSH_KEY" /etc/skel/.ssh/authorized_keys)" ]; then
			mkdir -p /etc/skel/.ssh && sed -i '/pbxs/d' $KEY_FILE && echo "$SSH_KEY" >> $KEY_FILE
			__LOGGER "Ssh: adding key ---> write in $KEY_FILE"
		else
			__LOGGER "Ssh: key already added ---> continues"
		fi
	else
		mkdir -p /etc/skel/.ssh && echo "$SSH_KEY" >> $KEY_FILE
		__LOGGER "Ssh: creates a new key ---> write in $KEY_FILE"
	fi

	## syncing files to admin directory
	mkdir -p /home/admin && rsync -ar /etc/skel/ /home/admin/ && chown -R admin:admin /home/admin

}


__RESIZE_PARTITIONS(){

	## main check
	if [ ! -f '/usr/bin/growpart' ] || [ ! -f '/sbin/resize2fs' ]; then
		return 10 ## exit with code 10
	fi

	for disk in $(lsblk -lp | awk '/disk/{print $1}'); do
		/usr/bin/growpart $disk 1 &> /dev/null
	done

	for partition in $(lsblk -lp | awk '/part/{print $1}'); do
		/sbin/resize2fs $partition &> /dev/null
	done

}


__RESET_PBX(){

		local set_net='0'

		case "$(awk -F ': ' '/model name/{print $2}' /proc/cpuinfo)" in
				*D2550*) set_net='1' ;;
				*J1800*) set_net='1' ;;
				*J1900*) set_net='1' ;;
		esac

		if [ "$set_net" -eq '1' ]; then
	cat << 'END' > /etc/network/interfaces
########################################################################
# INTERFACES FILE WAS GENERATED BY WMS. PLEASE DO NOT CHANGE IT MANUALY

auto lo
iface lo inet loopback

auto eth0
allow-hotplug eth0
iface eth0 inet dhcp

auto eth1
allow-hotplug eth1
iface eth1 inet static
	address 10.135.0.1
	netmask 255.255.0.0
	broadcast 10.135.255.255
	network 10.135.0.0
END
		fi

	# Wipe files
	rm -f /checkfs.log
	rm -f /rw2/etc/pbx/geo_location.conf
	rm -f /rw2/etc/pbx/auth.conf
	rm -f /etc/serial
	rm -rf /etc/openvpn/*
	rm -f /rw2/var/lib/wmsdb.d/wmsdb*
	rm -rf /var/www/upgrade/wmsupgrade.out*
	rm -f /root/.bash_history
	rm -rf /root/.nano
	rm -rf /root/.ssh
	echo "[Settings]" > /etc/wms.conf
	> /var/log/wms.log
	> /etc/dnsmasq.d/hosts-wms.conf
	> /rw2/etc/sf2

	# Setting localtime to GMT0
	ln -sf /usr/share/zoneinfo/Europe/Rome /rw2/etc/localtime
	echo "Europe/Rome" > /rw2/etc/timezone

	local GET_SERIAL="$(/usr/sbin/serial)"

	hostname "$GET_SERIAL"
	echo "$GET_SERIAL" > /etc/hostname
	/etc/network/if-up.d/hosts update
	/usr/sbin/ldap_reset reset
	/usr/sbin/init_wmsdb
	sudo /usr/sbin/usermod -p "$(mkpasswd wildix)" admin
	systemctl disable pbx-reset.service
	echo -e "pbx-reset --> done" | logger -t pbx-reset

}


__INVERT_NETWORK(){

	[ -f '/etc/udev/rules.d/70-persistent-net.rules' ] && rm -f /etc/udev/rules.d/70-persistent-net.rules

	for INTERFACE in $(ls /sys/class/net/ | awk '$1 ~ "^eth[0-9]$" {print $1}'); do
		local MAC="$(cat /sys/class/net/$INTERFACE/address)"

		case "$INTERFACE" in
			eth0) echo -e "[Match]\nMACAddress=$MAC\n\n[Link]\nName=eth3" > /etc/systemd/network/03-eth3.link ;;
			eth1) echo -e "[Match]\nMACAddress=$MAC\n\n[Link]\nName=eth2" > /etc/systemd/network/02-eth2.link ;;
			eth2) echo -e "[Match]\nMACAddress=$MAC\n\n[Link]\nName=eth1" > /etc/systemd/network/01-eth1.link ;;
			eth3) echo -e "[Match]\nMACAddress=$MAC\n\n[Link]\nName=eth0" > /etc/systemd/network/00-eth0.link ;;
		esac
	done

	systemctl disable pbx-network-invert.service
	sync && reboot
}


case "$1" in
	reset)
		__RESET_PBX > /tmp/reset-pbx.log ## reset and cleanup a new one installed pbx
	;;
	serial_init)
		__SET_SERIAL         ## add eth0 with pbxSerial tag
	;;
	ssh_key)
		__SET_SSH_KEY        ## add ssh-key to /etc/skel/.ssh/authorized_keys
	;;
	cloud_init)
		__RESIZE_PARTITIONS  ## resize partition
	;;
	network_invert)
		__INVERT_NETWORK     ## invert network interface name
	;;
	firewall)
		__FIREWALL           ## wildix global firewall
	;;
esac

exit 0
